/*
  This file is a part of hashutils package.
  Copyright (C) 2004 Zeljko Vrba.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  You can reach me at the following e-mail addresses:

  zvrba@globalnet.hr
  mordor@fly.srk.fer.hr
*/
#ifndef CDT_LINE_H__
#define CDT_LINE_H__

#include <stddef.h>
#include <cdt.h>
#include <sfio.h>
#include <vmalloc.h>

/**
   @file
   Implementation of intervals: we want to be able to output e.g. 1-3 instead
   of 1,2,3. For this we maintain an ordered set of half-open intervals
   [lb, ub). Intervals NEVER overlap; they are joined in one single interval.
   A single number n is represented by the interval [n,n+1).
*/

/** Interval structure, CDT and vmalloc disciplines. */
struct interval {
	Dtlink_t _l;				/* link */
	unsigned long bounds[2];	/* 0: lower, 1: upper bound */
};

/*
  Interval discipline for containers. Note that interval must ALWAYS be
  allocated before insertion in the container. The container make and free
  functions are always set to NULL.
*/
extern Dtdisc_t Cdt_IntervalDiscipline;

Dt_t *IntervalSet_create(void);
struct interval *IntervalSet_insert1(Dt_t *dt, unsigned long n);
struct interval *IntervalSet_insert2(Dt_t *dt, unsigned long lb, unsigned long ub);

#endif /* CDT_LINE_H__ */
