/*
  This file is a part of hashutils package.
  Copyright (C) 2004 Zeljko Vrba.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  You can reach me at the following e-mail addresses:

  zvrba@globalnet.hr
  mordor@fly.srk.fer.hr
*/
#ifndef CDT_LINE_H__
#define CDT_LINE_H__

#include <stddef.h>
#include <cdt.h>
#include <sfio.h>
#include <vmalloc.h>

/** Line hash structure, CDT and vmalloc discipline. */
struct line {
	Dtlink_t _l;				/* link */
	unsigned int n;				/* line number */
	char h[1];					/* hash */
};

/*
  Line discipline for containers. Note that line must ALWAYS be allocated
  before inserted in the container. The container make and free functions
  are always set to NULL.
*/
extern Dtdisc_t Cdt_LineDiscipline;

/*
  Ugly, but portable way to calculate minimum allocation size given
  the hash length in bytes (we have to obey struct member alignment).
  h begins at some offset; adding hlen to that offset gives the necessary
  size of struct.
*/
#define LINESZ(hlen) (offsetof(struct line, h) + hlen)

/* Read file lines into set. */
Dt_t *LineSet_read_file(Sfio_t *f, int halg);
extern int LineSet_report_collisions;
extern Vmalloc_t *LineSet_vmpool;
extern Vmdisc_t *Vmdcmmap;


#endif /* CDT_LINE_H__ */
