/*
  This file is a part of hashutils package.
  Copyright (C) 2004 Zeljko Vrba.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  You can reach me at the following e-mail addresses:

  zvrba@globalnet.hr
  mordor@fly.srk.fer.hr
*/
#include <mycrypt.h>
#include "cdt_line.h"

/**
  @file
  CDT discipline for set of lines.
*/

static char rcsid[] = "$Id: cdt_line_set.c,v 1.4 2004/02/15 12:29:10 zvrba Exp $";

/* TODO: nonportable. Is key always suitably aligned for unsigned int? */
static unsigned int line_hashf(Dt_t *dt, Void_t *key, Dtdisc_t *disc)
{
	return *(unsigned int*)key;
}

Dtdisc_t Cdt_LineDiscipline = {
	offsetof(struct line, h),	/* key */
	0,							/* key size: calculated depending on hash */
	0,							/* link: beginning of struct */
	NULL,						/* no copy method - must be allocated */
	NULL,						/* free */
	NULL,						/* compare: use CDT internal implementation */
	line_hashf,
	NULL,
	NULL
};

int LineSet_report_collisions = 1;
Vmalloc_t *LineSet_vmpool = NULL;

/**
   This function reads an already open file line by line and stores the hash
   of each line into data container (currently, unordered set).
*/
Dt_t *LineSet_read_file(Sfio_t *f, int halg)
{
	char *record;
	Dt_t *set;
	unsigned int recno = 0;

	if(!(set = dtopen(&Cdt_LineDiscipline, Dtset))) {
		sfprintf(sfstderr, "FATAL: memory exhausted. Exiting.\n");
		exit(1);
	}
	while((record = sfgetr(f, '\n', 0))) {
		unsigned long outlen = Cdt_LineDiscipline.size;
		struct line *line, *inserted;
		int err;

		if(!(line = vmalloc(LineSet_vmpool, LINESZ(Cdt_LineDiscipline.size)))) {
			sfprintf(sfstderr, "FATAL: memory exhausted. Exiting.\n");
			exit(1);
		}

		line->n = ++recno;
		if((err = hash_memory(halg, (unsigned char*)record,
							  sfvalue(f), line->h, &outlen)) != CRYPT_OK) {
			sfprintf(sfstderr, "FATAL: crypto error: %s, exiting.\n",
					 error_to_string(err));
			exit(1);
		}

		/* line is last allocated block so it can be freed even with Vmlast */
		if((inserted = dtinsert(set, line)) != line) {
			if(LineSet_report_collisions) {
				sfprintf(sfstderr, "WARNING: hash collision or duplicate lines: %d,%d\n", inserted->n, line->n);
			}
			vmfree(LineSet_vmpool, inserted);
		}
	}

	if(sferror(f)) {
		sfprintf(sfstderr, "ERROR reading file: ");
		perror("sfgetr");
		dtclose(set);
		set = NULL;
	}
	return set;
}
