/*
  This file is a part of hashutils package.
  Copyright (C) 2004 Zeljko Vrba.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  You can reach me at the following e-mail addresses:

  zvrba@globalnet.hr
  mordor@fly.srk.fer.hr
*/
#include <assert.h>
#include "halfhash.h"

static const char rcsid[] = "$Id: halfhash.c,v 1.4 2004/02/15 12:29:10 zvrba Exp $";

struct _hash_descriptor md2half_desc;
struct _hash_descriptor md4half_desc;
struct _hash_descriptor md5half_desc;
struct _hash_descriptor rmd128half_desc;

#define INIT_DESC(dst, src, Name, Id, Done)\
  assert(src.hashsize == 16);\
  memcpy(&dst, &src, sizeof(struct _hash_descriptor));\
  dst.name = Name; dst.ID = Id;\
  dst.hashsize = 8; dst.blocksize = src.blocksize;\
  dst.done = Done

#define DEFINE_DONE(name, srcdesc)\
static int name(hash_state *md, unsigned char *hash64)\
{\
  unsigned char hash128[16];\
  int i, err = srcdesc.done(md, hash128);\
  if(err == CRYPT_OK) for(i = 0; i < 8; i++) hash64[i] = hash128[i] ^ hash128[i+8];\
  return err;\
} 

DEFINE_DONE(md2half_done, md2_desc)
DEFINE_DONE(md4half_done, md4_desc)
DEFINE_DONE(md5half_done, md5_desc)
DEFINE_DONE(rmd128half_done, rmd128_desc)

void init_half_hashes(void)
{
	INIT_DESC(md2half_desc, md2_desc, "md2hx", 16, md2half_done);
	INIT_DESC(md4half_desc, md4_desc, "md4hx", 17, md4half_done);
	INIT_DESC(md5half_desc, md5_desc, "md5hx", 18, md5half_done);
	INIT_DESC(rmd128half_desc, rmd128_desc, "rmd128hx", 19, rmd128half_done);
}
