/*
  This file is a part of hashutils package.
  Copyright (C) 2004 Zeljko Vrba.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  You can reach me at the following e-mail addresses:

  zvrba@globalnet.hr
  mordor@fly.srk.fer.hr
*/

#if defined(_WIN32)
#include <windows.h>
#else
#include <unistd.h>
#include <sys/mman.h>
#endif

#include <vmalloc.h>

static char rcsid[] = "$Id: vmdcmmap.c,v 1.1 2004/03/18 18:21:09 zvrba Exp $";

/*
  A vmalloc discipline to get memory by anonymous mmap on UNIX or VirtualAlloc
  on WIN32. The WIN32 code is copied from AT&T's original vmbest.c.

  @todo Rewrite to use mmap of /dev/zero instead of anonymous mmaps. But only
  if other platforms don't support it.
*/
static Void_t *memoryf(
	Vmalloc_t *vm, Void_t *caddr,
	size_t csize, size_t nsize,
	Vmdisc_t *disc)
{
#if defined(_WIN32)
	if(csize == 0)
        return (Void_t*)VirtualAlloc(NIL(LPVOID),nsize,MEM_COMMIT,PAGE_READWRITE);
    if(nsize == 0)
        return VirtualFree((LPVOID)caddr,0,MEM_RELEASE) ? caddr : NIL(Void_t*);
    return NIL(Void_t*);
#else
	void *retval;

	/* old size 0; allocate new memory */
	if(csize == 0) {
		retval = mmap(NULL, nsize, PROT_READ | PROT_WRITE,
					  MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
		return retval == MAP_FAILED ? NULL : retval;
	}

	/* new size 0; free old memory */
	if(nsize == 0) {
		return munmap(caddr, csize) < 0 ? NULL : caddr;
	}

	/* mremap(2) is not portable; currently resize isn't supported */
	return NULL;
#endif
}

/*
  Our memory requirements are very large so declare to allocate memory in
  multiples of 64M = (1<<26).
*/
static Vmdisc_t _vmdcmmap = { memoryf, NULL, 1<<26 };
Vmdisc_t *Vmdcmmap = &_vmdcmmap;
